package com.zehong.web.controller.detector;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.util.BeanUtil;
import com.zehong.common.enums.ResultEnum;
import com.zehong.common.exception.IOCException;
import com.zehong.system.domain.TDetectorReportData;
import com.zehong.system.domain.TDetectorUser;
import com.zehong.system.domain.TDetectorInfo;
import com.zehong.system.domain.dto.TDetectorInfoDTO;
import com.zehong.system.domain.form.TDetectorReportDataForm;
import com.zehong.system.domain.vo.TDetectorAlarmInfoVO;
import com.zehong.system.domain.vo.TDetectorUserVO;
import com.zehong.system.service.ITDetectorReportDataService;
import com.zehong.system.service.ITDetectorUserService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.ModelMap;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.zehong.common.annotation.Log;
import com.zehong.common.core.controller.BaseController;
import com.zehong.common.core.domain.AjaxResult;
import com.zehong.common.enums.BusinessType;
import com.zehong.system.service.ITDetectorInfoService;
import com.zehong.common.utils.poi.ExcelUtil;
import com.github.pagehelper.PageInfo;
import com.zehong.common.core.page.TableDataInfo;

import javax.validation.Valid;

/**
 * 探测器Controller
 * 
 * @author zehong
 * @date 2021-11-02
 */
@RestController
@RequestMapping("/detector/detectorInfo")
@Slf4j
public class TDetectorInfoController extends BaseController
{
    @Autowired
    private ITDetectorInfoService tDetectorInfoService;
    @Autowired
    private ITDetectorUserService tDetectorUserService;
    @Autowired
    private ITDetectorReportDataService tDetectorReportDataService;

     /**
     * 查询探测器列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TDetectorInfo tDetectorInfo)
    {
        startPage();
        PageInfo<TDetectorInfo> list = tDetectorInfoService.selectTDetectorInfoPage(tDetectorInfo);
        return getDataTable(list);
    }

    /**
     * 获取探测器列表
     * @param tDetectorInfo
     * @return
     */
    @GetMapping("/detectorInfoList")
    public AjaxResult detectorInfoList(TDetectorInfo tDetectorInfo){
        return AjaxResult.success(tDetectorInfoService.selectTDetectorInfoList(tDetectorInfo));
    }

    /**
     * 导出探测器列表
     */
    @Log(title = "探测器", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(TDetectorInfo tDetectorInfo)
    {
        List<TDetectorInfo> list = tDetectorInfoService.selectTDetectorInfoList(tDetectorInfo);
        ExcelUtil<TDetectorInfo> util = new ExcelUtil<TDetectorInfo>(TDetectorInfo.class);
        return util.exportExcel(list, "探测器数据");
    }

    /**
     * 获取探测器详细信息
     */
    @GetMapping(value = "/{detectorId}")
    public AjaxResult getInfo(@PathVariable("detectorId") Long detectorId)
    {
        return AjaxResult.success(tDetectorInfoService.selectTDetectorInfoById(detectorId));
    }

    /**
     * 新增探测器
     */
    @Log(title = "探测器", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TDetectorInfo tDetectorInfo)
    {
        return toAjax(tDetectorInfoService.insertTDetectorInfo(tDetectorInfo));
    }

    /**
     * 修改探测器
     */
    @Log(title = "探测器", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TDetectorInfo tDetectorInfo)
    {
        return toAjax(tDetectorInfoService.updateTDetectorInfo(tDetectorInfo));
    }

    /**
     * 删除探测器
     */
    @Log(title = "探测器", businessType = BusinessType.DELETE)
	@DeleteMapping("/{detectorIds}")
    public AjaxResult remove(@PathVariable Long[] detectorIds)
    {
        return toAjax(tDetectorInfoService.deleteTDetectorInfoByIds(detectorIds));
    }

    @ApiOperation(value = "大屏数据初始化时主动查询的感知设备报警信息前50条")
    @GetMapping("/getAlarmData")
    public AjaxResult getAlarmData(){
        ModelMap map = new ModelMap();
        try {
            TDetectorInfo detector = new TDetectorInfo();
            detector.setIsDel("0");
            List<TDetectorInfo> tDetectorInfoList = tDetectorInfoService.selectTDetectorInfoList(detector);

            List<String> strList = new ArrayList<>();
            for(TDetectorInfo info : tDetectorInfoList){
                strList.add(info.getDetectorCode());
            }

            List<TDetectorReportData> dataList = tDetectorReportDataService.countDetector(strList);
            int totalNum = dataList.size();

            int handledNum = 0;
            for(TDetectorReportData data : dataList){
                if(!StringUtils.isEmpty(data.getIsCancelAlarm()) && !data.getIsCancelAlarm().equals("0")){
                    ++handledNum;
                }
            }

            List<TDetectorAlarmInfoVO> alarmVO = tDetectorReportDataService.selectTDetectorAlarm();
            for(TDetectorAlarmInfoVO alarm : alarmVO){
                if("1".equals(alarm.getDetectorType())){
                    alarm.setDetectorType("家用探测器");
                } else if ("2".equals(alarm.getDetectorType())){
                    alarm.setDetectorType("工业探测器");
                }
            }

            map.put("totalNum", totalNum);
            map.put("handledNum", handledNum);
            map.put("noHandeldNum", totalNum-handledNum);
            map.put("pageData", alarmVO);

        } catch (Exception ie){
            ie.printStackTrace();
            log.error("【查询报警数据】操作出错,error={}",ie.getMessage());
            throw new IOCException(ResultEnum.OPERATION_FAIL.getCode(),"查询报警数据操作出错");
        }

        return AjaxResult.success(map);
    }
}
